#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#define XK_LATIN1
#include <X11/keysymdef.h>


#define FIFOINNAME "trkOUTxmap"
#define FIFOOUTNAME "trkINxmap"
#define WMAPNAME "wmap.dat"
#define TRKDIR ".trk/"


Display      *display;
Window       window, root_win;
GC           gc;
Colormap     colormap;
XColor       color;
Drawable     drawable,  sat;
Widget       toplevel, frame;
XtAppContext appcontext;
Arg          warg[3];
Pixmap       pixmap, wmap;
Cursor       main_curs;
XEvent       event;
XWindowAttributes  winattr;
char         initflag=0;
XFontStruct  *fontsatname, *fontobsname;

int screen;
char appname[80], wmapfilename[80];
unsigned long numcolors, colorBLACK, colorWHITE, colorSEA, colorGND, colorLAKE,
  colorFOOTPRINT, colorSATNAME, colorOBSNAME;
unsigned int MAPWIDTH=600, MAPHEIGHT=400;

typedef struct satdata{
  char name[40];
  float lon;
  float lat;
  float height;
} SATDATA;

typedef struct obsdata{
  char name[40];
  float lon;
  float lat;
} OBSDATA;

void createcolors(void), drawmap(void), drawsat(float, float, float, char *),
  drawobs(float, float, char *);
int open_file(int *, int *, char *, int, char *);
void lonlat2xy(float, float, int *, int *);

int main(int argc, char** argv){
int clo, create=0, n, i, nsat=0, nobs=0, needupdate=1;
char dirpath[80]="\0", b[80], cmd[4], name[40];
int fdi, fdo, Xfd, lastfd;
fd_set rfds;
SATDATA sd[6];
OBSDATA od[6];
float lon, lat, height;

  /* Command line scan*/
  if(argc>1) {
    while(1){
      clo=getopt(argc, argv, "cd:h?");
      if(clo==-1) break;
      switch(clo){
      case 'c': create=1; break;
      case 'd': sscanf(optarg, "%s", dirpath); break;
      case 'h':
      case '?':
	printf("\ntrk X map:\n\n");
	printf(" -c create fifos if they don't exist \n");
	printf(" -d <dirpath> where the fifos are located (default ~/.trk)\n"
	       );
	return(0);
      }
    }
  }

  /* Open files */
  if(open_file(&fdi, &fdo, wmapfilename, create, dirpath)==-1) return 1;

  display = XOpenDisplay("");
  if (display == NULL) {
    fprintf(stderr, "cannot connect to X server \n");
    return(1);
  }
  /* X Server can be identified by a file descriptor */
  Xfd=ConnectionNumber(display);

  root_win=RootWindow(display, DefaultScreen(display));

  createcolors();

  window=XCreateSimpleWindow(display, root_win, 
			     0, 0,
			     MAPWIDTH, MAPHEIGHT,
			     10,
			     colorGND, colorSEA
			     );
  XMapWindow(display, window);  
  XStoreName(display, window, "trkXmap");
  main_curs=XCreateFontCursor(display, XC_X_cursor);
  XDefineCursor(display, window, main_curs);

  XFlush(display);

  gc=XCreateGC(display, window, 0, 0);

  fontsatname=XLoadQueryFont(display, "*helvetica*8*");
  if(!fontsatname) fprintf(stderr, "Unable to load font\n");
  fontobsname=XLoadQueryFont(display, "*helvetica*8*");
  if(!fontobsname) fprintf(stderr, "Unable to load font\n");

  XSelectInput(display, window, 
	       ExposureMask|KeyPressMask);

  drawmap();

  lastfd=fdi; if(Xfd>lastfd) lastfd=Xfd; lastfd++;
  FD_ZERO(&rfds); FD_SET(fdi, &rfds); FD_SET(fdo, &rfds); FD_SET(Xfd, &rfds);

  while(1){
    if(needupdate==1){
      XCopyArea(display,wmap, window, gc, 0, 0, MAPWIDTH, MAPHEIGHT, 0, 0);

      for(n=1; n<=nsat; n++)
	drawsat(sd[n-1].lon, sd[n-1].lat, sd[n-1].height, sd[n-1].name);
      for(n=1; n<=nobs; n++)
	drawobs(od[n-1].lon, od[n-1].lat, od[n-1].name);

      XFlush(display); 
      needupdate=0;
    }

    select(lastfd, &rfds, NULL, NULL, NULL);    

    /* X server */
    if(FD_ISSET(Xfd, &rfds)) {
      while(XPending(display)>0){
	XNextEvent(display, &event);
	
	switch (event.type) {
	case Expose:
	  XGetWindowAttributes(display, window, &winattr);
	  if(winattr.width!=MAPWIDTH || winattr.height!=MAPHEIGHT) {
	    MAPWIDTH=winattr.width; MAPHEIGHT=winattr.height;
	    drawmap();
	  }
	  needupdate=1; break;
	  
	  
	case KeyPress:
	  if(XKeycodeToKeysym(display, event.xkey.keycode,0)==XK_q) {
	    /* Exiting */
	    XUndefineCursor(display, window);	
	    XFreePixmap(display, wmap);
	    XCloseDisplay(display);
	    close(fdi); close(fdo);
	    return 0;
	  }
	  
	default:
	  continue;
	}
      }

      FD_ZERO(&rfds);
      FD_SET(fdi, &rfds); FD_SET(Xfd, &rfds); continue;
    }


    /* fifo */
    if(FD_ISSET(fdi, &rfds)) {
      n=read(fdi, b, 80);
      sscanf(b, "%s", cmd);

      if(strcasecmp(cmd, "s")==0){
	/* Add/update satellite */
	if(nsat<6){
	  nsat++;
	  sscanf(b, "%s %f %f %f %s", cmd, &lon, &lat, &height, name);
	  for(n=1; n<nsat; n++)
	    if(strcmp(name, sd[n-1].name)==0) { nsat--; break; }

	  sd[n-1].lon=lon; sd[n-1].lat=lat;
	  sd[n-1].height=height; strcpy(sd[n-1].name, name);

	  needupdate=1;
      	}

	else {
	strcpy(b, "Too many satellite!\n");
	write(fdo, b, strlen(b));
	}

      }

      else if(strcasecmp(cmd, "s-")==0){
	/* Remove satellite */
	if(nsat>0) {
	  /* First search for a satellite so named */
	  sscanf(b, "%s %s", cmd, name);
	  for(i=0; i<6; i++) 
	    if(strcmp(name, sd[i].name)==0) break;

	  /* Error message if satellite is not found */
	  if(i==6) {
	    strcpy(b, "No such satellite to remove!\n");
	    write(fdo, b, strlen(b));
	  }
	  else {	  
	    nsat--;                 /* Reduce the number of sattelite shown */
	    if(i<nsat){             /* And recompact data if needed */
	      for(n=i+1; n<=nsat; n++)
		sd[n-1]=sd[n];
	    }
	    needupdate=1;
	  }
	}
	else {
	  strcpy(b, "No satellite to remove!\n");
	  write(fdo, b, strlen(b));
	}
      }

      else if(strcasecmp(cmd, "o")==0){
	/* Add/update observer */
	if(nobs<6){
	  nobs++;
	  sscanf(b, "%s %f %f %s", cmd, &od[nobs-1].lon, &od[nobs-1].lat,
		 od[nobs-1].name);
	  for(n=1; n<nsat; n++)
	    if(strcmp(name, od[n-1].name)==0) { nsat--; break; }
	  needupdate=1;
      	}

	else {
	strcpy(b, "Too many observers!\n");
	write(fdo, b, strlen(b));
	}

      }

      else if(strcasecmp(cmd, "o-")==0){
	/* Remove observer */
	if(nobs>0) {
	  /* First search for an observer so named */
	  sscanf(b, "%s %s", cmd, name);
	  for(i=0; i<6; i++) 
	    if(strcmp(name, od[i].name)==0) break;

	  /* Error message if observer is not found */
	  if(i==6) {
	    strcpy(b, "No such observer to remove!\n");
	    write(fdo, b, strlen(b));
	  }
	  else {	  
	    nobs--;                 /* Reduce the number of observer shown */
	    if(i<nobs){             /* And recompact data if needed */
	      for(n=i+1; n<=nobs; n++)
		od[n-1]=od[n];
	    }
	    needupdate=1;
	  }
	}

	else {
	strcpy(b, "No observer to remove!\n");
	write(fdo, b, strlen(b));
	}
      }

      else if(strcasecmp(cmd, "q")==0) {
	/* Exiting */
	XUndefineCursor(display, window);	
	XFreePixmap(display, wmap);
	XCloseDisplay(display);
	close(fdi); close(fdo);
	return 0;
      }

      else if(strcasecmp(cmd, "r")==0) {
	/* Redraw Screen */
	nsat=0; nobs=0;
	needupdate=1;
      }
      
      else if(strcasecmp(cmd, "m")==0) {
	/* Minimize */
	XIconifyWindow(display, window, DefaultScreen(display));
	needupdate=1;
      }
      
      else if(strcasecmp(cmd, "e")==0) {
	/* Enlarge */
	XMapWindow(display, window);
	needupdate=1;
      }
      
      else {
	strcpy(b, "Wrong command to trkXmap\n");
	write(fdo, b, strlen(b));
      }
      
      FD_ZERO(&rfds);
      FD_SET(fdi, &rfds); FD_SET(Xfd, &rfds); continue;
    }

  }

}


void drawmap(void){
XPoint *vertex;
FILE *mfd;
long int l, ml;
int vx, vy;
double x, y;
char b[80];
XFontStruct *font1;

#define waitmsg "DRAWING WORLD MAP..."

 wmap=XCreatePixmap(display, DefaultRootWindow(display), MAPWIDTH, MAPHEIGHT, 
		    DefaultDepth(display, DefaultScreen(display)));

 XSetLineAttributes(display, gc,
		    1, LineSolid, CapButt, JoinBevel);
 
 XSetForeground(display, gc, colorSEA);
 XFillRectangle(display, wmap, gc, 0, 0, MAPWIDTH, MAPHEIGHT);

 font1=XLoadQueryFont(display, "*helvetica*24*");
 if(!font1) fprintf(stderr, "Unable to load font\n");
 XSetFont(display, gc, font1->fid);
 XSetForeground(display, gc, colorBLACK);
 XDrawString(display, window, gc,
	     (MAPWIDTH-XTextWidth(font1, waitmsg, strlen(waitmsg)))/2,
	     (MAPHEIGHT-font1->ascent-font1->descent)/2,
	     waitmsg, strlen(waitmsg));
 XFlush(display);

 if((mfd=fopen(wmapfilename, "r"))==NULL) 
   { printf("Unable to find map file\n"); return; }
 
 ml=-1; l=0;
 do{
   fgets(b, 78, mfd);
   if(b[0]=='T' || b[0]=='L') { if(l>ml) {ml=l;  l=0;} }
   else l++;
 }while(!feof(mfd));
 
 if((vertex=(XPoint*) calloc(ml, sizeof(XPoint)))==NULL)
   { printf("Unable to allocate memory for map data\n"); return; }
 
 rewind(mfd);
 
 
 l=0; XSetForeground(display, gc, colorGND);
 do{
   fgets(b, 78, mfd);
   if(b[0]=='T' && l!=0){
     XFillPolygon(display, wmap, gc, vertex, l, Nonconvex, CoordModeOrigin);
     l=0;
   }
   else if(b[0]=='L') { l=0; continue; }
   else {
     if(sscanf(b, "%lf %lf", &x, &y)!=2) continue;
     vertex[l].x=((float)x/360.+.5)*MAPWIDTH; 
     vertex[l].y=(.5-(float)y/180.)*MAPHEIGHT;
     l++;
   }
 }while(!feof(mfd));
 
 rewind(mfd);
 
 l=0; XSetForeground(display, gc, colorLAKE);
 do{
   fgets(b, 78, mfd);
   if(b[0]=='L' && l!=0){
     XFillPolygon(display, wmap, gc, vertex, l, Nonconvex, CoordModeOrigin);
     l=0;
   }
   else if(b[0]=='T') { l=0; continue; }
   else {
     if(sscanf(b, "%lf %lf", &x, &y)!=2) continue;
     lonlat2xy(x, y, &vx, &vy);
     vertex[l].x=vx; vertex[l].y=vy;
     l++;
   }
 }while(!feof(mfd));
 fclose(mfd);
 
 free(vertex);
}


void drawsat(float lon, float lat, float h, char *name){
int x, y, xp, yp, xpo, ypo;
float t, cosa, sina, fplon, fplat;

#define INCR 0.05
#define R        6378.135
 
 XSetFont(display, gc, fontsatname->fid);
 XSetForeground(display, gc, colorFOOTPRINT);
 XSetForeground(display, gc, colorSATNAME);


 lonlat2xy(lon, lat, &x, &y);
 XDrawString(display, window, gc,
	     x-XTextWidth(fontsatname, name, strlen(name))/2,
	     y-(fontsatname->descent-fontsatname->ascent)/2,
	     name, strlen(name));

 lon=lon/180.*M_PI; lat=lat/180.*M_PI;

 cosa=R/(R+h); sina=sqrt(1-cosa*cosa);

 xpo=-MAPWIDTH; ypo=-MAPHEIGHT;
 for(t=0.; t<=2*M_PI+INCR; t+=INCR){
   fplat=asin(cos(t)*cos(lat)*sina+sin(lat)*cosa);
   fplon=acos((cosa-sin(lat)*sin(fplat))/(cos(lat)*cos(fplat)));
   if(t<M_PI) fplon=fplon+lon;
   else fplon=lon-fplon;
   if(isnan(fplon)!=0 || isnan(fplat)!=0) continue;
   lonlat2xy(fplon*180./M_PI, fplat*180./M_PI, &xp, &yp);
   if(abs(xpo-xp)<MAPWIDTH/2 && abs(ypo-yp)<MAPHEIGHT/2)
     XDrawLine(display, window, gc, xpo, ypo, xp, yp);
   xpo=xp; ypo=yp;
 }

}

void drawobs(float lon, float lat, char *name){  
int x, y;

 XSetFont(display, gc, fontobsname->fid);

 lonlat2xy(lon, lat, &x, &y);

 XSetForeground(display, gc, colorOBSNAME);
 XDrawString(display, window, gc,
	     x-XTextWidth(fontobsname, name, strlen(name))/2,
	     y-(fontobsname->descent-fontobsname->ascent)/2+6,
	     name, strlen(name));

 XSetLineAttributes(display, gc, 2, LineSolid, CapRound, JoinRound);

 XFillArc(display, window, gc, x-3, y-2, 6, 4, 0, 360*64);
}

/*
  Transform geodetical coordinate to point in the map: this function
  choose projection system
*/
void lonlat2xy(float lon, float lat, int *x, int *y){

  while(lon>180.) lon-=360.; while(lon<-180.) lon+=360.;
  while(lat>180.) lat-=360.; while(lat<-180.) lat+=360.;
  *x=(lon/360.+.5)*MAPWIDTH;
  *y=(.5-lat/180.)*MAPHEIGHT;
}

void createcolors(void){
  
  colormap=DefaultColormap(display, screen);
  color.flags=DoRed|DoGreen|DoBlue;
  
  /* black */
  color.red=0; color.green=0; color.blue=0;
  XAllocColor(display, colormap, &color);
  colorBLACK=color.pixel;
  
  /* white */
  color.red=65535; color.green=65535; color.blue=65535;
  XAllocColor(display, colormap, &color);
  colorWHITE=color.pixel;
  
  /* sea */
  color.red=0; color.green=1000; color.blue=50000;
  XAllocColor(display, colormap, &color);
  colorSEA=color.pixel;
  
  /* ground */
  color.red=1000; color.green=50000; color.blue=0;
  XAllocColor(display, colormap, &color);
  colorGND=color.pixel;
  
  /* lake */
  color.red=0; color.green=1000; color.blue=40000;
  XAllocColor(display, colormap, &color);
  colorLAKE=color.pixel;

  /* footprint */
  color.red=40000; color.green=0; color.blue=20000;
  XAllocColor(display, colormap, &color);
  colorFOOTPRINT=color.pixel;

  /* satellite name */
  color.red=60000; color.green=20000; color.blue=0;
  XAllocColor(display, colormap, &color);
  colorSATNAME=color.pixel;

  /* observer name */
  color.red=60000; color.green=60000; color.blue=60000;
  XAllocColor(display, colormap, &color);
  colorOBSNAME=color.pixel;
}


int open_file(int *fdi, int *fdo, char *wmapfn, int create, char *path){

int mode;
char HomeDir[40], cfgdir[40], gf[80];

 if(path[0]=='\0') {
   mode=F_OK; strcpy(HomeDir, getenv("HOME"));
   if(HomeDir==NULL) {
     printf("Unable to find HOME dir ?!\n"); return -1;
   }
   strcat(HomeDir, "/");
   strcpy(cfgdir, HomeDir); strcat(cfgdir, TRKDIR);
 }
 else {
   strcpy(cfgdir, path);
   if(cfgdir[strlen(cfgdir)-1]!='/') strcat(cfgdir, "/");
 }

 if(access(cfgdir, mode)==-1) {
   printf("Unable to find a \"%s\" dir\n", cfgdir);
   return -1;
 }   
 
 mode=F_OK; strcpy(gf, cfgdir); strcat(gf, FIFOINNAME);
 if(access(gf, mode)==-1){
   if(create==0) {
     printf("Unable to find \"%s\"\n", gf); return -1;
   }
   else {
     if(mkfifo(gf, 0600)==-1) {
       fprintf(stderr, "Unable to create input fifo \"%s\"\n", gf);
       return -1;
     }
   }
 }

 if((*fdi=open(gf, O_RDWR | O_NDELAY))==-1) {
   fprintf(stderr, "Unable to open input fifo \"%s\"\n", gf);
   return -1;
 }

 mode=F_OK; strcpy(gf, cfgdir); strcat(gf, FIFOOUTNAME);
 if(access(gf, mode)==-1){
   if(create==0) {
     printf("Unable to find \"%s\"\n", gf); return -1;
   }
   else {
     if(mkfifo(gf, 0600)==-1) {
       fprintf(stderr, "Unable to create output fifo \"%s\"\n", gf);
       return -1;
     }
   }
 }

 if((*fdo=open(gf, O_RDWR | O_NDELAY))==-1) {
   fprintf(stderr, "Unable to open output fifo \"%s\"\n", gf);
   return -1;
 }
 
 mode=F_OK; strcpy(gf, cfgdir); strcat(gf, WMAPNAME);
 if(access(gf, mode)==-1){
   printf("Unable to find \"%s\"\n", gf); return -1;
 }
 strcpy(wmapfn, gf);

 return 0;
}
