/*
 * TRK - Satellite tracking program based on Norad SGP/SDP model with
 *       curses interface
 *
 *	by Lapo Pieri IK5NAX  2000-2011
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send bugs reports, comments, critique, etc, to ik5nax@amsat.org
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include "tleflt.h"


int main(int argc, char **argv){
  int clo, rv, lc, i;
  FILE *flt, *in;
  char elem[3][81], *fltit, linebuff[81], *lb;

  if(argc>1){
    while(1){
      clo=getopt(argc, argv, "hv");
      if(clo==-1) break;
      switch(clo){
      case 'h': 
	usage(); return 0;
      case 'v':
	printf("tleflt version %s\n", __VERSION);
	return 0;
      }
    }
        
    /* first of all store flt */
    
    if((flt=fopen(argv[optind], "r"))==NULL){
      printf("Unable to open %s\n", argv[optind]);
      return 1;
    }
    lc=0;
    while(fgets(linebuff, 80, flt)!=NULL)
      lc++;
    rewind(flt);
    fltit=(char *)malloc(lc*81*sizeof(char));
    for(i=0; i<lc; i++){
      fgets(linebuff, 80, flt);
      lb=linebuff;
      while(lb[0]==' ' || lb[0]=='\t') lb++; /* strip leading spcs & tabs */
      if(lb[0]=='#')                         /* ignore comment lines */
	continue;
      if(strlen(lb)==0)                      /* ignore blank lines */
	continue;
      /* strcpy(fltit[81*i], lb); */
      while(lb[strlen(lb)-1]=='\n' || lb[strlen(lb)-1]=='\r')
	lb[strlen(lb)-1]='\0'; 
      strcpy(fltit+(81*i), lb);
    }
    
    for(i=0; i<lc; i++){
      printf("%s\n", fltit+i*81);
    }
    
    while(argc-optind>1){
      optind++;      
      if((in=fopen(argv[optind], "r"))==NULL){
	printf("Unable to open %s\n", argv[optind]);
	return 1;
      }
      while((rv=getelem(in, elem))!=-1){
	if(rv==0){
	  for(i=0; i<lc; i++)
	    if(strstr(elem[0], fltit+i*81)!=NULL)
	      break;
	  if(i<lc)
	    printf("%s\n%s\n%s\n", elem[0], elem[1], elem[2]);
	}
	else
	  printf("internal error: getelm returns %d\n", rv);
      }
      fclose(in);
    }

  }
  else
    usage();

  return 0;
}

int getelem(FILE *fd, char elem[][81]){
  char linebuff[81], *lb;
  int chkl=-1;

  while(1){
    if(fgets(linebuff, 80, fd)==NULL)
      return -1;
    lb=linebuff;
    while(lb[0]==' ' || lb[0]=='\t') lb++;   /* strip leading spcs & tabs */
    if(lb[0]=='#')                           /* ignore comment lines */
      continue;
    if(strlen(lb)==0)                        /* ignore blank lines */
      continue;
    if(lb[0]=='1'){
      if(chkl!=0)
	return 1;                            /* line '1' readed but in the
						wrong place */    
    }
    else if(lb[0]=='2'){
      if(chkl!=1)
	return 2;                            /* line '2' readed but in the
						wrong place */
    }
    else{
      if(chkl!=-1)
	return 3;                            /* line '0' readed but in the
						wrong place */
    }
    chkl++;
    strcpy(elem[chkl], lb);
    if((lb=strchr(elem[chkl], '\n'))!=NULL)
      *lb='\0';
    if((lb=strchr(elem[chkl], '\r'))!=NULL)
      *lb='\0';

    if(chkl==2)
      return 0;
  }

}


void usage(void){
  printf("tleflt  <flt> <infile1> [infile2]...\n");

}
